@@ -1,7 +1,10 @@
-from .base_requests import AnymailRequestsBackend, RequestsPayload
+from django.conf import settings
+from django.utils.encoding import force_str
+
 from ..exceptions import AnymailRequestsAPIError
 from ..message import AnymailRecipientStatus
 from ..utils import get_anymail_setting, update_deep
+from .base_requests import AnymailRequestsBackend, RequestsPayload
 
 
 class EmailBackend(AnymailRequestsBackend):
@@ -13,12 +16,18 @@ class EmailBackend(AnymailRequestsBackend):
 
     def __init__(self, **kwargs):
         """Init options from Django settings"""
-        self.api_key = get_anymail_setting('api_key', esp_name=self.esp_name,
-                                           kwargs=kwargs, allow_bare=True)
-        self.subaccount = get_anymail_setting('subaccount', esp_name=self.esp_name,
-                                              kwargs=kwargs, default=None)
-        api_url = get_anymail_setting('api_url', esp_name=self.esp_name, kwargs=kwargs,
-                                      default="https://api.sparkpost.com/api/v1/")
+        self.api_key = get_anymail_setting(
+            "api_key", esp_name=self.esp_name, kwargs=kwargs, allow_bare=True
+        )
+        self.subaccount = get_anymail_setting(
+            "subaccount", esp_name=self.esp_name, kwargs=kwargs, default=None
+        )
+        api_url = get_anymail_setting(
+            "api_url",
+            esp_name=self.esp_name,
+            kwargs=kwargs,
+            default="https://api.sparkpost.com/api/v1/",
+        )
         if not api_url.endswith("/"):
             api_url += "/"
         super().__init__(api_url, **kwargs)
@@ -34,9 +43,13 @@ class EmailBackend(AnymailRequestsBackend):
             rejected = results["total_rejected_recipients"]
             transmission_id = results["id"]
         except (KeyError, TypeError) as err:
-            raise AnymailRequestsAPIError("Invalid SparkPost API response format",
-                                          email_message=message, payload=payload,
-                                          response=response, backend=self) from err
+            raise AnymailRequestsAPIError(
+                "Invalid SparkPost API response format",
+                email_message=message,
+                payload=payload,
+                response=response,
+                backend=self,
+            ) from err
 
         # SparkPost doesn't (yet*) tell us *which* recipients were accepted or rejected.
         # (* looks like undocumented 'rcpt_to_errors' might provide this info.)
@@ -44,45 +57,52 @@ class EmailBackend(AnymailRequestsBackend):
         # else just report 'unknown' for all recipients.
         recipient_count = len(payload.recipients)
         if accepted == recipient_count and rejected == 0:
-            status = 'queued'
+            status = "queued"
         elif rejected == recipient_count and accepted == 0:
-            status = 'rejected'
+            status = "rejected"
         else:  # mixed results, or wrong total
-            status = 'unknown'
-        recipient_status = AnymailRecipientStatus(message_id=transmission_id, status=status)
-        return {recipient.addr_spec: recipient_status for recipient in payload.recipients}
+            status = "unknown"
+        recipient_status = AnymailRecipientStatus(
+            message_id=transmission_id, status=status
+        )
+        return {
+            recipient.addr_spec: recipient_status for recipient in payload.recipients
+        }
 
 
 class SparkPostPayload(RequestsPayload):
     def __init__(self, message, defaults, backend, *args, **kwargs):
         http_headers = {
-            'Authorization': backend.api_key,
-            'Content-Type': 'application/json',
+            "Authorization": backend.api_key,
+            "Content-Type": "application/json",
         }
         if backend.subaccount is not None:
-            http_headers['X-MSYS-SUBACCOUNT'] = backend.subaccount
+            http_headers["X-MSYS-SUBACCOUNT"] = backend.subaccount
         self.recipients = []  # all recipients, for backend parse_recipient_status
         self.cc_and_bcc = []  # for _finalize_recipients
-        super().__init__(message, defaults, backend, headers=http_headers, *args, **kwargs)
+        super().__init__(
+            message, defaults, backend, headers=http_headers, *args, **kwargs
+        )
 
     def get_api_endpoint(self):
         return "transmissions/"
 
     def serialize_data(self):
         self._finalize_recipients()
+        self._check_content_options()
         return self.serialize_json(self.data)
 
     def _finalize_recipients(self):
         # https://www.sparkpost.com/docs/faq/cc-bcc-with-rest-api/
-        # self.data["recipients"] is currently a list of all to-recipients. We need to add
-        # all cc and bcc recipients. Exactly how depends on whether this is a batch send.
+        # self.data["recipients"] is currently a list of all to-recipients. Must add all
+        # cc and bcc recipients. Exactly how depends on whether this is a batch send.
         if self.is_batch():
             # For batch sends, must duplicate the cc/bcc for *every* to-recipient
             # (using each to-recipient's metadata and substitutions).
             extra_recipients = []
             for to_recipient in self.data["recipients"]:
                 for email in self.cc_and_bcc:
-                    extra = to_recipient.copy()  # capture "metadata" and "substitutions", if any
+                    extra = to_recipient.copy()  # gets "metadata" and "substitutions"
                     extra["address"] = {
                         "email": email.addr_spec,
                         "header_to": to_recipient["address"]["header_to"],
@@ -94,17 +114,46 @@ class SparkPostPayload(RequestsPayload):
             # "To" header to show all the "To" recipients...
             full_to_header = ", ".join(
                 to_recipient["address"]["header_to"]
-                for to_recipient in self.data["recipients"])
+                for to_recipient in self.data["recipients"]
+            )
             for recipient in self.data["recipients"]:
                 recipient["address"]["header_to"] = full_to_header
             # ... and then simply add the cc/bcc to the end of the list.
             # (There is no per-recipient data, or it would be a batch send.)
             self.data["recipients"].extend(
-                {"address": {
-                    "email": email.addr_spec,
-                    "header_to": full_to_header,
-                }}
-                for email in self.cc_and_bcc)
+                {
+                    "address": {
+                        "email": email.addr_spec,
+                        "header_to": full_to_header,
+                    }
+                }
+                for email in self.cc_and_bcc
+            )
+
+    # SparkPost silently ignores certain "content" payload fields
+    # when a template_id is used.
+    IGNORED_WITH_TEMPLATE_ID = {
+        # SparkPost API content.<field> -> feature name (for error message)
+        "attachments": "attachments",
+        "inline_images": "inline images",
+        "headers": "extra headers and/or cc recipients",
+        "from": "from_email",
+        "reply_to": "reply_to",
+    }
+
+    def _check_content_options(self):
+        if "template_id" in self.data["content"]:
+            # subject, text, and html will cause 422 API Error:
+            #     "message": "Both content object and template_id are specified",
+            #     "code": "1301"
+            # but others are silently ignored in a template send:
+            ignored = [
+                feature_name
+                for field, feature_name in self.IGNORED_WITH_TEMPLATE_ID.items()
+                if field in self.data["content"]
+            ]
+            if ignored:
+                self.unsupported_feature("template_id with %s" % ", ".join(ignored))
 
     #
     # Payload construction
@@ -118,7 +167,8 @@ class SparkPostPayload(RequestsPayload):
         }
 
     def set_from_email(self, email):
-        self.data["content"]["from"] = email.address
+        if email:
+            self.data["content"]["from"] = email.address
 
     def set_to(self, emails):
         if emails:
@@ -127,11 +177,14 @@ class SparkPostPayload(RequestsPayload):
             # (We use "header_to" rather than "name" to simplify some logic
             # in _finalize_recipients; the results end up the same.)
             self.data["recipients"].extend(
-                {"address": {
-                    "email": email.addr_spec,
-                    "header_to": email.address,
-                }}
-                for email in emails)
+                {
+                    "address": {
+                        "email": email.addr_spec,
+                        "header_to": email.address,
+                    }
+                }
+                for email in emails
+            )
             self.recipients += emails
 
     def set_cc(self, emails):
@@ -155,7 +208,9 @@ class SparkPostPayload(RequestsPayload):
 
     def set_reply_to(self, emails):
         if emails:
-            self.data["content"]["reply_to"] = ", ".join(email.address for email in emails)
+            self.data["content"]["reply_to"] = ", ".join(
+                email.address for email in emails
+            )
 
     def set_extra_headers(self, headers):
         if headers:
@@ -166,7 +221,8 @@ class SparkPostPayload(RequestsPayload):
 
     def set_html_body(self, body):
         if "html" in self.data["content"]:
-            # second html body could show up through multiple alternatives, or html body + alternative
+            # second html body could show up through multiple alternatives,
+            # or html body + alternative
             self.unsupported_feature("multiple html parts")
         self.data["content"]["html"] = body
 
@@ -179,19 +235,27 @@ class SparkPostPayload(RequestsPayload):
             super().add_alternative(content, mimetype)
 
     def set_attachments(self, atts):
-        attachments = [{
-            "name": att.name or "",
-            "type": att.content_type,
-            "data": att.b64content,
-        } for att in atts if not att.inline]
+        attachments = [
+            {
+                "name": att.name or "",
+                "type": att.content_type,
+                "data": att.b64content,
+            }
+            for att in atts
+            if not att.inline
+        ]
         if attachments:
             self.data["content"]["attachments"] = attachments
 
-        inline_images = [{
-            "name": att.cid,
-            "type": att.mimetype,
-            "data": att.b64content,
-        } for att in atts if att.inline]
+        inline_images = [
+            {
+                "name": att.cid,
+                "type": att.mimetype,
+                "data": att.b64content,
+            }
+            for att in atts
+            if att.inline
+        ]
         if inline_images:
             self.data["content"]["inline_images"] = inline_images
 
@@ -208,6 +272,36 @@ class SparkPostPayload(RequestsPayload):
             if to_email in merge_metadata:
                 recipient["metadata"] = merge_metadata[to_email]
 
+    def set_merge_headers(self, merge_headers):
+        def header_var(field):
+            return "Header__" + field.title().replace("-", "_")
+
+        merge_header_fields = set()
+
+        for recipient in self.data["recipients"]:
+            to_email = recipient["address"]["email"]
+            if to_email in merge_headers:
+                recipient_headers = merge_headers[to_email]
+                recipient.setdefault("substitution_data", {}).update(
+                    {header_var(key): value for key, value in recipient_headers.items()}
+                )
+                merge_header_fields.update(recipient_headers.keys())
+
+        if merge_header_fields:
+            headers = self.data.setdefault("content", {}).setdefault("headers", {})
+            # Global substitution_data supplies defaults for defined headers:
+            self.data.setdefault("substitution_data", {}).update(
+                {
+                    header_var(field): headers[field]
+                    for field in merge_header_fields
+                    if field in headers
+                }
+            )
+            # Indirect merge_headers through substitution_data:
+            headers.update(
+                {field: "{{%s}}" % header_var(field) for field in merge_header_fields}
+            )
+
     def set_send_at(self, send_at):
         try:
             start_time = send_at.replace(microsecond=0).isoformat()
@@ -229,13 +323,22 @@ class SparkPostPayload(RequestsPayload):
 
     def set_template_id(self, template_id):
         self.data["content"]["template_id"] = template_id
-        # Must remove empty string "content" params when using stored template
+        # Must remove empty string "content" params when using stored template.
+        # (Non-empty params are left in place, to cause API error.)
         for content_param in ["subject", "text", "html"]:
             try:
                 if not self.data["content"][content_param]:
                     del self.data["content"][content_param]
             except KeyError:
                 pass
+        # "from" is also silently ignored. Strip it if empty or DEFAULT_FROM_EMAIL,
+        # else leave in place to cause error in _check_content_options.
+        try:
+            from_email = self.data["content"]["from"]
+            if not from_email or from_email == force_str(settings.DEFAULT_FROM_EMAIL):
+                del self.data["content"]["from"]
+        except KeyError:
+            pass
 
     def set_merge_data(self, merge_data):
         for recipient in self.data["recipients"]:
